import type { InferResultType } from "@/utils/drizzle";

type UsagePerDay = {
	day: number; // 0 (Sunday) to 6 (Saturday)
	usageMinutes: number;
	dataPoints: number;
};

function aggregateUsageByDay(jobs: InferResultType<"printJobs">[]): UsagePerDay[] {
	const usagePerDayMap = new Map<number, number>();
	const usagePerDayDatapointsMap = new Map<number, number>();

	for (const job of jobs) {
		let remainingDuration = job.durationInMinutes;
		const currentStart = new Date(job.startAt);

		while (remainingDuration > 0) {
			const day = currentStart.getDay();
			const dataPoints = usagePerDayDatapointsMap.get(day) || 0;
			usagePerDayDatapointsMap.set(day, dataPoints + 1);

			// Calculate minutes remaining in the current day
			const minutesRemainingInDay = (24 - currentStart.getHours()) * 60 - currentStart.getMinutes();
			const minutesToAdd = Math.min(remainingDuration, minutesRemainingInDay);

			// Update the usage for the current day
			const usageMinutes = usagePerDayMap.get(day) || 0;
			usagePerDayMap.set(day, usageMinutes + minutesToAdd);

			// Update remaining duration and move to the next day
			remainingDuration -= minutesToAdd;
			currentStart.setDate(currentStart.getDate() + 1);
			currentStart.setHours(0, 0, 0, 0); // Start at the beginning of the next day
		}
	}

	const usageData: UsagePerDay[] = Array.from({ length: 7 }, (_, day) => ({
		day,
		usageMinutes: usagePerDayMap.get(day) || 0,
		dataPoints: usagePerDayDatapointsMap.get(day) || 0,
	}));

	return usageData;
}

export function forecastPrinterUsage(jobs: InferResultType<"printJobs">[]): number[] {
	const usageData = aggregateUsageByDay(jobs);
	const forecasts: number[] = [];
	for (const data of usageData) {
		let usagePrediction = data.usageMinutes / data.dataPoints;
		if (Number.isNaN(usagePrediction)) {
			usagePrediction = 0;
		}
		forecasts.push(Math.round(usagePrediction));
	}

	return forecasts;
}
